EXPORTS

    ; DirectShow filter exports (standard DLL functions)
    DllGetClassObject        PRIVATE
    DllCanUnloadNow          PRIVATE
    DllRegisterServer        PRIVATE
    DllUnregisterServer      PRIVATE

    ; basic functions
    CompressFile
    DecompressFile
    ConvertFile
    VerifyFile
    CompressFileW
    DecompressFileW
    ConvertFileW
    VerifyFileW
    CompressFileW2
    DecompressFileW2
    ConvertFileW2
    VerifyFileW2

    ; interface wrappers
    c_APEDecompress_Create
    c_APEDecompress_CreateW
    c_APEDecompress_Destroy
    c_APEDecompress_GetData
    c_APEDecompress_Seek
    c_APEDecompress_GetInfo
    c_APEDecompress_SetNumberOfThreads

    c_APECompress_Create
    c_APECompress_Destroy
    c_APECompress_Start
    c_APECompress_StartW
    c_APECompress_AddData
    c_APECompress_GetBufferBytesAvailable
    c_APECompress_LockBuffer
    c_APECompress_UnlockBuffer
    c_APECompress_Finish
    c_APECompress_SetNumberOfThreads

    ; helpers / miscellaneous
    GetVersionNumber
    GetInterfaceCompatibility
    GetLibraryVersionString
    GetLibraryVersionNumber
    GetLibraryInterfaceVersion
    RemoveTag
    TagFileSimple
    GetID3Tag
    FillWaveHeader
    FillWaveFormatEx
    
    ; Winamp
    winampGetInModule2
    winampGetExtendedFileInfo

    ; CoolEdit
    QueryCoolFilter
    OpenFilterOutput
    CloseFilterOutput
    WriteFilterOutput
    OpenFilterInput
    CloseFilterInput
    ReadFilterInput
    FilterUnderstandsFormat
    FilterGetOptions
    FilterOptions
    FilterGetFileSize
    FilterOptionsString
    DIALOGMsgProc
    FilterGetFirstSpecialData
    FilterGetNextSpecialData
    FilterWriteSpecialData
