#pragma once

/**************************************************************************************************
Warnings
**************************************************************************************************/
#ifdef _MSC_VER

#pragma warning(disable:4263)
#pragma warning(disable:4264)
#pragma warning(disable:4265)
#pragma warning(disable:4266)
#pragma warning(disable:4365)
#pragma warning(disable:4555)
#pragma warning(disable:4596)
#pragma warning(disable:4619)
#pragma warning(disable:4625)
#pragma warning(disable:4626)
#pragma warning(disable:4668)
#pragma warning(disable:4703)
#pragma warning(disable:4711)
#pragma warning(disable:4746)
#pragma warning(disable:4820)
#pragma warning(disable:4865)
#pragma warning(disable:4996)
#pragma warning(disable:5026)
#pragma warning(disable:5027)
#pragma warning(disable:5038)
#pragma warning(disable:5039)
#pragma warning(disable:5045)
#pragma warning(disable:5204)
#pragma warning(disable:5220)
#pragma warning(disable:5246)
#pragma warning(disable:5266)
#pragma warning(disable:5267)
#pragma warning(disable:6001)
#pragma warning(disable:6011)
#pragma warning(disable:6031)
#pragma warning(disable:6064)
#pragma warning(disable:6067)
#pragma warning(disable:6101)
#pragma warning(disable:6273)
#pragma warning(disable:6284)
#pragma warning(disable:6302)
#pragma warning(disable:6328)
#pragma warning(disable:6384)
#pragma warning(disable:6385)
#pragma warning(disable:6386)
#pragma warning(disable:6387)
#pragma warning(disable:6509)
#pragma warning(disable:26444)
#pragma warning(disable:26495)
#pragma warning(disable:28159)
#pragma warning(disable:28183)
#pragma warning(disable:28196)
#pragma warning(disable:28251)
#pragma warning(disable:28252)
#pragma warning(disable:28253)

#endif
